## $Id: interfaces.py 9006 2012-07-16 07:36:13Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Customized interfaces of the university application package.
"""

from zope import schema
from waeup.kofa.applicants.interfaces import (
    IApplicantBaseData,
    AppCatCertificateSource, CertificateSource)
from waeup.kofa.schoolgrades import ResultEntryField
from waeup.kofa.interfaces import (
    SimpleKofaVocabulary, academic_sessions_vocab, validate_email)
from waeup.kofa.schema import FormattedDate, TextLineChoice
from waeup.kofa.students.vocabularies import nats_vocab, GenderSource
from waeup.kofa.applicants.interfaces import contextual_reg_num_source
from kofacustom.nigeria.applicants.interfaces import (
    LGASource, high_qual, high_grade, exam_types,
    INigeriaUGApplicant, INigeriaPGApplicant,
    INigeriaApplicantOnlinePayment,
    INigeriaUGApplicantEdit, INigeriaPGApplicantEdit,
    INigeriaApplicantUpdateByRegNo,
    IPUTMEApplicantEdit,
    UG_OMIT_DISPLAY_FIELDS,
    UG_OMIT_PDF_FIELDS,
    UG_OMIT_MANAGE_FIELDS,
    UG_OMIT_EDIT_FIELDS,
    PUTME_OMIT_EDIT_FIELDS,
    UG_OMIT_RESULT_SLIP_FIELDS,
    PG_OMIT_DISPLAY_FIELDS,
    PG_OMIT_PDF_FIELDS,
    PG_OMIT_MANAGE_FIELDS,
    PG_OMIT_EDIT_FIELDS,
    )
from waeup.uniben.interfaces import MessageFactory as _
from waeup.uniben.payments.interfaces import ICustomOnlinePayment

############################################################
# Use former lgas for Uniben. Must be removed after application.
from waeup.uniben.applicants.lgas import LGAS
class LGASource(LGASource):
    """A source for school subjects used in exam documentation.
    """
    lga_dict = dict(LGAS)
############################################################

class ICustomUGApplicant(INigeriaUGApplicant):
    """An undergraduate applicant.

    This interface defines the least common multiple of all fields
    in ug application forms. In customized forms, fields can be excluded by
    adding them to the UG_OMIT* tuples.
    """

    nationality = schema.Choice(
        source = nats_vocab,
        title = _(u'Nationality'),
        required = False,
        )
    lga = schema.Choice(
        source = LGASource(),
        title = _(u'State/LGA (Nigerians only)'),
        required = False,
        )
    perm_address = schema.Text(
        title = _(u'Permanent Address'),
        required = False,
        )
    course1 = schema.Choice(
        title = _(u'1st Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = True,
        )
    course2 = schema.Choice(
        title = _(u'2nd Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = False,
        )
    jamb_subjects = schema.Text(
        title = _(u'Subjects and Scores'),
        required = False,
        )
    jamb_score = schema.Int(
        title = _(u'Total Score'),
        required = False,
        )
    notice = schema.Text(
        title = _(u'Notice'),
        required = False,
        )
    screening_venue = schema.TextLine(
        title = _(u'Screening Venue'),
        required = False,
        )
    screening_date = schema.TextLine(
        title = _(u'Screening Date'),
        required = False,
        )
    screening_score = schema.Int(
        title = _(u'Screening Score (%)'),
        required = False,
        )
    aggregate = schema.Int(
        title = _(u'Aggregate Score (%)'),
        description = _(u'(average of relative JAMB and PUTME scores)'),
        required = False,
        )
    result_uploaded = schema.Bool(
        title = _(u'Result uploaded'),
        default = False,
        )
    student_id = schema.TextLine(
        title = _(u'Student Id'),
        required = False,
        readonly = False,
        )
    course_admitted = schema.Choice(
        title = _(u'Admitted Course of Study'),
        source = CertificateSource(),
        required = False,
        )
    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        )

class ICustomPGApplicant(INigeriaPGApplicant):
    """A postgraduate applicant.

    This interface defines the least common multiple of all fields
    in pg application forms. In customized forms, fields can be excluded by
    adding them to the PG_OMIT* tuples.
    """

    nationality = schema.Choice(
        source = nats_vocab,
        title = _(u'Nationality'),
        required = False,
        )
    lga = schema.Choice(
        source = LGASource(),
        title = _(u'State/LGA (Nigerians only)'),
        required = False,
        )
    perm_address = schema.Text(
        title = _(u'Permanent Address'),
        required = False,
        )
    course1 = schema.Choice(
        title = _(u'1st Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = True,
        )
    course2 = schema.Choice(
        title = _(u'2nd Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = False,
        )
    hq_type = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = high_qual,
        )
    hq_matric_no = schema.TextLine(
        title = _(u'Former Matric Number'),
        required = False,
        readonly = False,
        )
    hq_degree = schema.Choice(
        title = _(u'Class of Degree'),
        required = False,
        readonly = False,
        vocabulary = high_grade,
        )
    hq_school = schema.TextLine(
        title = _(u'Institution Attended'),
        required = False,
        readonly = False,
        )
    hq_session = schema.TextLine(
        title = _(u'Years Attended'),
        required = False,
        readonly = False,
        )
    hq_disc = schema.TextLine(
        title = _(u'Discipline'),
        required = False,
        readonly = False,
        )
    pp_school = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = exam_types,
        )
    #presently = schema.Bool(
    #    title = _(u'Attending'),
    #    required = False,
    #    readonly = False,
    #    )
    presently_inst = schema.TextLine(
        title = _(u'If yes, name of institution'),
        required = False,
        readonly = False,
        )
    nysc_year = schema.Int(
        title = _(u'Nysc Year'),
        required = False,
        readonly = False,
        )
    nysc_lga = schema.Choice(
        source = LGASource(),
        title = _(u'Nysc Location'),
        required = False,
        )
    employer = schema.TextLine(
        title = _(u'Employer'),
        required = False,
        readonly = False,
        )
    emp_position = schema.TextLine(
        title = _(u'Employer Position'),
        required = False,
        readonly = False,
        )
    emp_start = FormattedDate(
        title = _(u'Start Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    emp_end = FormattedDate(
        title = _(u'End Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    emp_reason = schema.TextLine(
        title = _(u'Reason for Leaving'),
        required = False,
        readonly = False,
        )
    employer2 = schema.TextLine(
        title = _(u'2nd Employer'),
        required = False,
        readonly = False,
        )
    emp2_position = schema.TextLine(
        title = _(u'2nd Employer Position'),
        required = False,
        readonly = False,
        )
    emp2_start = FormattedDate(
        title = _(u'Start Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    emp2_end = FormattedDate(
        title = _(u'End Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    emp2_reason = schema.TextLine(
        title = _(u'Reason for Leaving'),
        required = False,
        readonly = False,
        )
    notice = schema.Text(
        title = _(u'Notice'),
        required = False,
        readonly = False,
        )
    student_id = schema.TextLine(
        title = _(u'Student Id'),
        required = False,
        readonly = False,
        )
    course_admitted = schema.Choice(
        title = _(u'Admitted Course of Study'),
        source = CertificateSource(),
        required = False,
        readonly = False,
        )
    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        )

class ICustomApplicant(ICustomUGApplicant, ICustomPGApplicant):
    """An interface for both types of applicants.

    Attention: The ICustomPGApplicant field seetings will be overwritten
    by ICustomPGApplicant field settings. If a field is defined
    in both interfaces zope.schema validates only against the
    constraints in ICustomUGApplicant. This does not affect the forms
    since they are build on either ICustomUGApplicant or ICustomPGApplicant.
    """

    def writeLogMessage(view, comment):
        """Adds an INFO message to the log file
        """

    def createStudent():
        """Create a student object from applicant data
        and copy applicant object.
        """

class ICustomUGApplicantEdit(ICustomUGApplicant):
    """An undergraduate applicant interface for edit forms.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )
    date_of_birth = FormattedDate(
        title = _(u'Date of Birth'),
        required = True,
        show_year = True,
        )

ICustomUGApplicantEdit[
    'date_of_birth'].order =  ICustomUGApplicant['date_of_birth'].order
ICustomUGApplicantEdit[
    'email'].order =  ICustomUGApplicant['email'].order

class ICustomPGApplicantEdit(ICustomPGApplicant):
    """A postgraduate applicant interface for editing.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )
    date_of_birth = FormattedDate(
        title = _(u'Date of Birth'),
        required = True,
        show_year = True,
        )

ICustomPGApplicantEdit[
    'date_of_birth'].order =  ICustomPGApplicant['date_of_birth'].order
ICustomPGApplicantEdit[
    'email'].order =  ICustomPGApplicant['email'].order

class ICustomApplicantOnlinePayment(INigeriaApplicantOnlinePayment):
    """An applicant payment via payment gateways.

    """

class IPUTMEApplicantEdit(ICustomUGApplicant):
    """An undergraduate applicant interface for editing.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """
    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )
    date_of_birth = FormattedDate(
        title = _(u'Date of Birth'),
        required = True,
        show_year = True,
        )

IPUTMEApplicantEdit[
    'date_of_birth'].order =  ICustomUGApplicant['date_of_birth'].order
IPUTMEApplicantEdit[
    'email'].order =  ICustomUGApplicant['email'].order

class ICustomApplicantUpdateByRegNo(ICustomApplicant):
    """Representation of an applicant.

    Skip regular reg_number validation if reg_number is used for finding
    the applicant object.
    """
    reg_number = schema.TextLine(
        title = u'Registration Number',
        required = False,
        )
